# analysis_bs_headons

This repository contains the scripts with which the plots shown in
"Boson star head-on collision with constraint violating and satisfying initial data"
were generated.

The scripts can be found inside `bs_headons`.
In `bs_headons/postprocess` there are scripts to extrapolate bamps' `Psi4` data to
null infinity and to compute the radiated GW energy as discussed in the paper.

## Installation

0. Install Julia
  - Either download from `https://julialang.org/downloads`
  - Or use the `juliaup` version manager: `https://github.com/JuliaLang/juliaup`
1. Clone this repository.
2. Clone `GWanalysis.jl` into a separate directory from `https://git.tpi.uni-jena.de/fatteneder/GWanalysis.jl`
   Check out the version `ef2127891642a7d0c348417299bf8d40e085243` which is needed for this repository to work.
3. Add `GWanalysis.jl` as a dependency:
   ```
   $ cd <this-dir>
   $ julia --project
   $ julia> using Pkg
   $ julia> Pkg.dev("/path/to/GWanalysis.jl")
   ```
4. Install dependencies:
  ```
  $ cd <this-dir>
  $ julia --project
  julia> using Pkg
  julia> Pkg.resolve()
  julia> Pkg.instantiate()
  ```
5. Adjust the variable `ROOTDIR` in `bs_headons/rootdirs.jl` so that it points to the
   folder that contains all the bamps results with the subfolders `comparison4, comparison8, etc.`.
   To get a grip of this data please ask Bernd Bruegmann, as I will let him know where I archive
   the data.
6. Run a plotting script:
  ```
  julia> include("bs_headons/dEdt_E/plot.jl")
  ```

## Tips & tricks

- Try keeping the interactive REPL alive and rerun the scripts with `include`.
- Install `Revise.jl` if you want to make on-the-fly changes to `GWanalysis.jl`
  (e.g. without restarting the Juila REPL):
  ```
  $ julia
  $ julia> using Pkg
  $ julia> Pkg.add("Revise")
  $ julia> using Revise # run this in every fresh REPL session before you use GWanalysis.jl
  $ julia> include("bs_headons/dEdt_E/plot.jl")
  $ julia> # edit something in GWanalysis.jl
  $ julia> include("bs_headons/dEdt_E/plot.jl") # changes should be picked up automatically
  ```

---

## Brief description of the plotting scripts

Figure numbers refer to the arxiv version of the paper: https://arxiv.org/pdf/2311.16251

- `amax_nocts-vs-cts`: Generates Fig 6
- `cmon_bdry_effect`: Generates Fig 7, the heatmap is generated using Paraview
- `cmon_nocts-vs-cts`: Generates Fig 2 & 3
- `cmon_rsc_amax_nocts-vs-cts`: Generates Fig 5
- `dEdt_comparison`: Generates Fig 14
- `dEdt_E`: Generates Fig 13
- `longrun`: Generates Fig 10
- `N_M_Madm_comparison`: Generates Fig 8 & 9
- `postprocess`: Scripts to process the Psi4 data from bamps output, their output is stored
  in folders called `analyze` in the respective bamps output directories.
- `psi4_comparison_cts_noncts`: Generates Fig 11
- `psi4_dhdt_h_problem`: Generates Fig 16 for each of the bamps outputs we have.
- `psi4_self_convergence`: Generates Fig 12
- `psi4_wiener`: Generates Fig 15
