using GWanalysis
using DataFrames
using GLMakie
import GLMakie.Makie.GeometryBasics: coordinates
using CairoMakie
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


#######################################################################
#                         Plotting functions                          #
#######################################################################


# get_xlims(ax::Axis) = @lift ($(ax.finallimits).origin[1], $(ax.finallimits).origin[1] + $(ax.finallimits).widths[1])
get_ylims(ax::Axis) = @lift ($(ax.finallimits).origin[2], $(ax.finallimits).origin[2] + $(ax.finallimits).widths[2])


function plot_output_0d!(ax, df, var, MBBS;
        resolutions=nothing, use_color=true,
        use_log=false, use_diff=false, use_norm_t0=false)

    if !isnothing(resolutions)
        @assert resolutions isa Vector{<:Int}
        df = filter(df) do row
            row.n in resolutions
        end
        @assert nrow(df) > 0
    end

    has_vars = [ var in out.output_0d for out in df.output ]
    @assert all(has_vars)

    all_ns   = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df.output ]

    ts_Psi, Psis = GWanalysis.get_output_0d(df, "max/ana.AmpPsi")
    _, idx = findmax(Psis[end])
    t_merger = ts_Psi[end][idx]

    trafo = identity
    if use_log
        trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs
    end
    norm = if use_norm_t0
        v -> v[1]
    else
        v -> 1.0
    end

    vec_ts, vec_vars = get_output_0d(df, var)

    if use_diff
        for i = 1:length(all_ns)-1
            t1, t2 = vec_ts[i], vec_ts[i+1]
            y1, y2 = vec_vars[i], vec_vars[i+1]

            ys_intrp, t_comm = GWanalysis.interpolate_common_grid([t1,t2], [y1,y2],
                                                                  idx_use_as_ref=2)
            absdiff_y = abs.(ys_intrp[2] .- ys_intrp[1])

            n1, n2 = all_ns[i], all_ns[i+1]
            lbl = L"\Delta n = %$n2 - %$n1"
            ls = isodd(i) ? :solid : :dash
            if use_color
                lines!(ax, t_comm, trafo.(absdiff_y ./ MBBS); label=lbl, linestyle=ls)
            else
                lines!(ax, t_comm, trafo.(absdiff_y ./ MBBS); label=lbl, color=(:black,0), linestyle=ls)
            end
        end
    else
        for (i,(t, var)) in enumerate(zip(vec_ts, vec_vars))
            n = all_ns[i]
            lbl = L"n = %$n"
            if use_color
                lines!(ax, t, trafo.(var./norm(var)); label=lbl)
            else
                lines!(ax, t, trafo.(var./norm(var)); label=lbl, color=(:black,0))
            end
        end
    end

    vlines!(ax, t_merger, color=:black, linestyle=:dash)

end


function plot_output_sphere!(ax, df, var, radius, MBBS;
        resolutions=nothing, use_color=true,
        use_log=false, use_diff=false, use_norm_t0=false)

    if !isnothing(resolutions)
        @assert resolutions isa Vector{<:Int}
        df = filter(df) do row
            row.n in resolutions
        end
        @assert nrow(df) > 0
    end

    has_vars = [ var in out.output_spheres for out in df.output ]
    @assert all(has_vars)

    all_ns   = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df.output ]

    ts_Psi, Psis = GWanalysis.get_output_0d(df, "max/ana.AmpPsi")
    _, idx = findmax(Psis[end])
    t_merger = ts_Psi[end][idx]

    trafo = identity
    if use_log
        trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs
    end
    norm = if use_norm_t0
        v -> v[1]
    else
        v -> 1.0
    end

    vec_rs, vec_ts, vec_vars = GWanalysis.get_output_spheres2(df, var)

    if use_diff
        for i = 1:length(all_ns)-1
            rs1, rs2 = vec_rs[i], vec_rs[i+1]
            t1, t2 = vec_ts[i], vec_ts[i+1]
            ys1, ys2 = vec_vars[i], vec_vars[i+1]

            i1 = findfirst(==(radius), rs1) |> something
            i2 = findfirst(==(radius), rs2) |> something

            ys_intrp, t_comm = GWanalysis.interpolate_common_grid([t1,t2], [ys1[i1],ys2[i2]],
                                                                  idx_use_as_ref=2)
            absdiff_y = abs.(ys_intrp[2] .- ys_intrp[1])

            n1, n2 = all_ns[i], all_ns[i+1]
            lbl = L"\Delta n = %$n2 - %$n1"
            ls = isodd(i) ? :solid : :dash
            if use_color
                lines!(ax, t_comm, trafo.(absdiff_y ./ MBBS); label=lbl, linestyle=ls)
            else
                lines!(ax, t_comm, trafo.(absdiff_y ./ MBBS); label=lbl, color=(:black,0), linestyle=ls)
            end
        end
    else
        for i = 1:length(all_ns)
            rs, t, ys = vec_rs[i], vec_ts[i], vec_vars[i]

            ii = findfirst(==(radius), rs) |> something

            n = all_ns[i]
            lbl = L"n = %$n"
            if use_color
                lines!(ax, t, ys[ii]./norm(ys[ii]); label=lbl)
            else
                lines!(ax, t, ys[ii]./norm(ys[ii]); label=lbl, color=(:black,0))
            end
        end
    end

    vlines!(ax, t_merger, color=:black, linestyle=:dash)

end


GLMakie.closeall()

let

ylims_cmon = (-17, -1)
yticks_cmon = -16:4:2
n = 13
max_n = 19
plt_w, plt_h = 1200, 500
MBBS = 0.79
radius = 720
plt_w, plt_h = 1200, 1800

output = find_runs(OUTPUTS["comparison8/cts_helfer_v_0.05"])
df = load_runs(output)

df.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df.output ]
filter!(df) do row
    row.n <= max_n
end


### Plot time evolution of 0d and sphere output normalized wrt initial data value

fig = Figure()
ga = fig[1,1] = GridLayout()


ax_N    = Axis(ga[1,1]; xlabel=L"t", ylabel=L"\frac{N(t)}{N(0)}")
ax_M    = Axis(ga[2,1]; xlabel=L"t", ylabel=L"\frac{M_{\textrm{tot}}(t)}{M_{\textrm{tot}}(0)}")
ax_Madm = Axis(ga[3,1]; xlabel=L"t", ylabel=L"\frac{M_{\textrm{ADM}}(t)}{M_{\textrm{ADM}}(0)}")

plot_output_0d!(ax_N, df, "integral/ana.noetherQintegrand", MBBS,
                resolutions=[n],
                use_log=false, use_diff=false, use_norm_t0=true)
plot_output_0d!(ax_M, df, "integral/ana.totalMintegrand", MBBS,
                resolutions=[n],
                use_log=false, use_diff=false, use_norm_t0=true)
plot_output_sphere!(ax_Madm, df, "ana.admMintegrand", radius, MBBS,
                    resolutions=[n],
                    use_log=false, use_diff=false, use_norm_t0=true)

# hack in an axis on the RHS so that we can plot the absolute values
dummy_ax_N    = Axis(ga[1,1]; yticksmirrored=true, yaxisposition=:right,
                     ylabel=L"N(t)")
dummy_ax_M    = Axis(ga[2,1]; yticksmirrored=true, yaxisposition=:right,
                     ylabel=L"M_{\textrm{tot}}(t)")
dummy_ax_Madm = Axis(ga[3,1]; yticksmirrored=true, yaxisposition=:right,
                     ylabel=L"M_{\textrm{ADM}}(t)")
# plot_output_0d!(dummy_ax_N, df, "integral/ana.noetherQintegrand",
#                 resolutions=[19],
#                 use_log=false, use_diff=false, use_norm_t0=true,
#                 use_color=false)
# plot_output_0d!(dummy_ax_M, df, "integral/ana.totalMintegrand",
#                 resolutions=[19],
#                 use_log=false, use_diff=false, use_norm_t0=true,
#                 use_color=false)
# plot_output_sphere!(dummy_ax_Madm, df, "ana.admMintegrand", radius,
#                     resolutions=[19],
#                     use_log=false, use_diff=false, use_norm_t0=true,
#                     use_color=false)

hidexdecorations!(ax_N, ticks=false, grid=false, minorgrid=false, minorticks=false)
hidexdecorations!(ax_M, ticks=false, grid=false, minorgrid=false, minorticks=false)
linkyaxes!(ax_N, ax_M, ax_Madm)
# linkyaxes!(dummy_ax_N, dummy_ax_M, dummy_ax_Madm)
linkxaxes!(ax_N, ax_M, ax_Madm, dummy_ax_N, dummy_ax_M, dummy_ax_Madm)

# manually extract the reference values at t=0 for run with n=19 (needed for rescaling of tick values)
subdf = filter(df) do row
    row.n == n
end
_, vec_Ns = get_output_0d(df, "integral/ana.noetherQintegrand")
_, vec_Ms = get_output_0d(df, "integral/ana.totalMintegrand")
vec_rs, _, vec_Madms = GWanalysis.get_output_spheres2(df, "ana.admMintegrand")
ii = findfirst(==(radius), vec_rs[1]) |> something
N0, M0, Madm0 = vec_Ns[1][1], vec_Ms[1][1], vec_Madms[1][ii][1]
@show(N0,M0,Madm0)

onany(ax_N.finallimits, ax_N.yaxis.tickvalues) do _, tvals
    ylims!(dummy_ax_N, get_ylims(ax_N)[].*N0)
    dummy_ax_N.yticks[] = (tvals.*N0, [ @sprintf("%.3f", ti .* N0) for ti in tvals ])
end
onany(ax_M.finallimits, ax_M.yaxis.tickvalues) do _, tvals
    ylims!(dummy_ax_M, get_ylims(ax_M)[].*M0)
    dummy_ax_M.yticks[] = (tvals.*M0, [ @sprintf("%.3f", ti .* M0) for ti in tvals ])
end
onany(ax_Madm.finallimits, ax_Madm.yaxis.tickvalues) do _, tvals
    ylims!(dummy_ax_Madm, get_ylims(ax_Madm)[].*Madm0)
    dummy_ax_Madm.yticks[] = (tvals.*Madm0, [ @sprintf("%.3f", ti .* Madm0) for ti in tvals ])
end
notify(ax_N.finallimits)
notify(ax_M.finallimits)
notify(ax_Madm.finallimits)
hidexdecorations!(dummy_ax_N)
hidexdecorations!(dummy_ax_M)
hidexdecorations!(dummy_ax_Madm)
# # leave ydecorations for dummy axis on in order to verify that the newly computed ticks
# # align with the ones from the other axis!!
# hideydecorations!(dummy_ax_N, label=false, ticklabels=false)
# hideydecorations!(dummy_ax_M, label=false, ticklabels=false)
# hideydecorations!(dummy_ax_Madm, label=false, ticklabels=false)
hidespines!(dummy_ax_N)
hidespines!(dummy_ax_M)
hidespines!(dummy_ax_Madm)

ax_N.ylabelpadding[]    = 20
ax_M.ylabelpadding[]    = 20
ax_Madm.ylabelpadding[] = 20
dummy_ax_N.ylabelpadding[]    = 10
dummy_ax_M.ylabelpadding[]    = 10
dummy_ax_Madm.ylabelpadding[] = 10

Legend(ga[0,1], ax_N, nbanks=2, labelvalign=:top, tellwidth=false, width=nothing,
       orientation=:horizontal)

rowgap!(ga, 0)


### Plot self convergence study

ga = fig[2,1] = GridLayout()

ax_N = Axis(ga[1,1], xlabel=L"t", ylabel=L"\log_{10} |\frac{\Delta N(t)}{M_{\textrm{BBS}}}|",
            ylabelsize=34)
plot_output_0d!(ga[1,1], df, "integral/ana.noetherQintegrand", MBBS,
                use_log=true, use_diff=true, use_norm_t0=false)
ax_M = Axis(ga[2,1], xlabel=L"t", ylabel=L"\log_{10} |\frac{\Delta M_{\textrm{tot}}(t)}{M_{\textrm{BBS}}}|",
            ylabelsize=34)
plot_output_0d!(ax_M, df, "integral/ana.totalMintegrand", MBBS,
                use_log=true, use_diff=true, use_norm_t0=false)
ax_Madm = Axis(ga[3,1], xlabel=L"t", ylabel=L"\log_{10} |\frac{\Delta M_{\textrm{ADM}}(t)}{M_{\textrm{BBS}}}|",
               ylabelsize=34)
plot_output_sphere!(ax_Madm, df, "ana.admMintegrand", radius, MBBS,
                    use_log=true, use_diff=true, use_norm_t0=false)

hidexdecorations!(ax_N, ticks=false, grid=false, minorgrid=false, minorticks=false)
hidexdecorations!(ax_M, ticks=false, grid=false, minorgrid=false, minorticks=false)
linkyaxes!(ax_N, ax_M, ax_Madm)
linkxaxes!(ax_N, ax_M, ax_Madm)

ax_N.ylabelpadding[]    = 40
ax_M.ylabelpadding[]    = 40
ax_Madm.ylabelpadding[] = 40

Legend(ga[0,1], ax_N, nbanks=2, labelvalign=:top, tellwidth=false, width=nothing,
       orientation=:horizontal, labelsize=35)

rowgap!(ga, 0)

plotname = joinpath(@__DIR__, "comparison8_cts_helfer_v_0.05_N_M_Madm_combined.v1$(EXT)")
@info "saving '$plotname'"
save(plotname, fig, resolution=(plt_w, plt_h), backend=CairoMakie)

display(fig)

end
