using GWanalysis
using DataFrames
using GLMakie
using CairoMakie
using Printf
GLMakie.activate!()


include("../rootdirs.jl")
include("../theme.jl")


#######################################################################
#                         Plotting functions                          #
#######################################################################


function plot_N_M_diffs(df_n, df_cn, df_h, df_ch, radius, M, ylims, yticks)

    #### fetch data

    var = "integral/ana.noetherQintegrand"
    t_N_n, N_n   = get_output_0d(df_n, var)
    t_N_cn, N_cn = get_output_0d(df_cn, var)
    t_N_h, N_h   = get_output_0d(df_h, var)
    t_N_ch, N_ch = get_output_0d(df_ch, var)

    var = "integral/ana.totalMintegrand"
    t_Mtot_n,  Mtot_n  = get_output_0d(df_n, var)
    t_Mtot_cn, Mtot_cn = get_output_0d(df_cn, var)
    t_Mtot_h,  Mtot_h  = get_output_0d(df_h, var)
    t_Mtot_ch, Mtot_ch = get_output_0d(df_ch, var)

    var = "ana.admMintegrand"
    rs_Madm_n,  t_Madm_n,  vec_Madm_n  = GWanalysis.get_output_spheres2(df_n, var)
    rs_Madm_cn, t_Madm_cn, vec_Madm_cn = GWanalysis.get_output_spheres2(df_cn, var)
    rs_Madm_h,  t_Madm_h,  vec_Madm_h  = GWanalysis.get_output_spheres2(df_h, var)
    rs_Madm_ch, t_Madm_ch, vec_Madm_ch = GWanalysis.get_output_spheres2(df_ch, var)

    n_idx  = findfirst(r -> r == radius, rs_Madm_n[1]) |> something
    cn_idx = findfirst(r -> r == radius, rs_Madm_cn[1]) |> something
    h_idx  = findfirst(r -> r == radius, rs_Madm_h[1]) |> something
    ch_idx = findfirst(r -> r == radius, rs_Madm_ch[1]) |> something
    Madm_n  = vec_Madm_n[1][n_idx]
    Madm_cn = vec_Madm_cn[1][cn_idx]
    Madm_h  = vec_Madm_h[1][h_idx]
    Madm_ch = vec_Madm_ch[1][ch_idx]

    t_Psi_n, Psi_n   = GWanalysis.get_output_0d(df_n, "max/ana.AmpPsi")
    t_Psi_cn, Psi_cn = GWanalysis.get_output_0d(df_cn, "max/ana.AmpPsi")
    t_Psi_h, Psi_h   = GWanalysis.get_output_0d(df_h, "max/ana.AmpPsi")
    t_Psi_ch, Psi_ch = GWanalysis.get_output_0d(df_ch, "max/ana.AmpPsi")

    # find merger time from maximum of scalar field amplitude
    _, idx_n = findmax(Psi_n[1]); t_merger_n = t_Psi_n[1][idx_n]
    _, idx_cn = findmax(Psi_cn[1]); t_merger_cn = t_Psi_cn[1][idx_cn]
    _, idx_h = findmax(Psi_h[1]); t_merger_h = t_Psi_h[1][idx_h]
    _, idx_ch = findmax(Psi_ch[1]); t_merger_ch = t_Psi_ch[1][idx_ch]
    t_merger_n = 0.5 * (t_merger_n + t_merger_cn)
    t_merger_h = 0.5 * (t_merger_h + t_merger_ch)

    # interpolate onto common grids
    N_n_intrp, t_N_n_comm       = GWanalysis.interpolate_common_grid([t_N_n[1], t_N_cn[1]], [N_n[1], N_cn[1]])
    N_h_intrp, t_N_h_comm       = GWanalysis.interpolate_common_grid([t_N_h[1], t_N_ch[1]], [N_h[1], N_ch[1]])
    Mtot_n_intrp, t_Mtot_n_comm = GWanalysis.interpolate_common_grid([t_Mtot_n[1], t_Mtot_cn[1]], [Mtot_n[1], Mtot_cn[1]])
    Mtot_h_intrp, t_Mtot_h_comm = GWanalysis.interpolate_common_grid([t_Mtot_h[1], t_Mtot_ch[1]], [Mtot_h[1], Mtot_ch[1]])
    Madm_n_intrp, t_Madm_n_comm = GWanalysis.interpolate_common_grid([t_Madm_n[1], t_Madm_cn[1]], [Madm_n, Madm_cn])
    Madm_h_intrp, t_Madm_h_comm = GWanalysis.interpolate_common_grid([t_Madm_h[1], t_Madm_ch[1]], [Madm_h, Madm_ch])

    #### plot

    fig = Figure()
    gl = fig[1,1] = GridLayout()
    axN    = Axis(gl[1,1]; xlabel=L"t", ylabel=L"\log_{10}|\frac{\Delta N}{M_{\textrm{BBS}}}|")
    axMtot = Axis(gl[2,1]; xlabel=L"t", ylabel=L"\log_{10}|\frac{\Delta M_{\textrm{tot}}}{M_{\textrm{BBS}}}|")
    axMadm = Axis(gl[3,1]; xlabel=L"t", ylabel=L"\log_{10}|\frac{\Delta M_{\textrm{ADM}}}{M_{\textrm{BBS}}}|")
    axN.yticks[]    = yticks
    axMtot.yticks[] = yticks
    axMadm.yticks[] = yticks
    linkaxes!(axN, axMtot, axMadm)
    ylims!(axN, ylims)
    ylims!(axMtot, ylims)
    ylims!(axMadm, ylims)
    hidexdecorations!(axN, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(axMtot, ticks=false, grid=false, minorgrid=false, minorticks=false)
    trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs

    vlines!(axN, t_merger_n, color=:black, linestyle=:dash)
    lines!(axN, t_N_n_comm, trafo.((@. N_n_intrp[1] - N_n_intrp[2])); label="SSP vs. CTS+SSP")
    lines!(axN, t_N_h_comm, trafo.((@. N_h_intrp[1] - N_h_intrp[2])); label="CVE vs. CTS+CVE")

    vlines!(axMadm, t_merger_n, color=:black, linestyle=:dash)
    lines!(axMadm, t_Madm_n_comm, trafo.((@. Madm_n_intrp[1] - Madm_n_intrp[2])); label="SSP vs. CTS+SSP")
    lines!(axMadm, t_Madm_h_comm, trafo.((@. Madm_h_intrp[1] - Madm_h_intrp[2])); label="CVE vs. CTS+CVE")

    vlines!(axMtot, t_merger_n, color=:black, linestyle=:dash)
    lines!(axMtot, t_Mtot_n_comm, trafo.((@. Mtot_n_intrp[1] - Mtot_n_intrp[2])); label="SSP vs. CTS+SSP")
    lines!(axMtot, t_Mtot_h_comm, trafo.((@. Mtot_h_intrp[1] - Mtot_h_intrp[2])); label="CVE vs. CTS+CVE")


    tmpgl = GridLayout(gl[3,1],
                    tellwidth = false, tellheight = false,
                    halign = :right, valign = :bottom)
    box = Box(tmpgl[1, 1], color = :white, strokecolor = :black)
    lbl = Label(tmpgl[1, 1], L"r_{\textrm{ext}} = %$radius", padding = (10,10,10,10) , fontsize=30)
    translate!(box.blockscene, (0,0,100))
    translate!(lbl.blockscene, (0,0,100))

    Legend(gl[0,1], axN, orientation=:horizontal, width=nothing, tellwidth=false)
    rowgap!(gl, 0)

    return fig

end


#######################################################################
#                               Script                                #
#######################################################################


let

radius = 720
M_BBS = 0.79
n = 11 # resolution
plt_w, plt_h = 1200, 1000
ylims = ()
yticks = -8:2:0
ylims = (-7,-1)

plotname = joinpath(@__DIR__, "N_M_diffs_comparison8_v_0.05_n_11$(EXT)")

runs_n  = find_runs(OUTPUTS["comparison8/naive_v_0.05"])
runs_h  = find_runs(OUTPUTS["comparison8/helfer_v_0.05"])
runs_cn = find_runs(OUTPUTS["comparison8/cts_naive_v_0.05"])
runs_ch = find_runs(OUTPUTS["comparison8/cts_helfer_v_0.05"])

df_n  = load_runs(runs_n)
df_h  = load_runs(runs_h)
df_cn = load_runs(runs_cn)
df_ch = load_runs(runs_ch)

df_n.n  = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_n.output ]
df_h.n  = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_h.output ]
df_cn.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_cn.output ]
df_ch.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_ch.output ]

for df in (df_n, df_h, df_cn, df_ch)
    filter!(df) do row
        row.n == n
    end
end

### compare for helfer/equal-mass-fixed superposition

fig = plot_N_M_diffs(df_n, df_cn, df_h, df_ch, radius, M_BBS, ylims, yticks)

display(fig)

@info "saving '$plotname'"
save(plotname, fig, resolution=(plt_w, plt_h), backend=CairoMakie)


end
