using GWanalysis
using DataFrames
using GLMakie
import GLMakie.Makie.GeometryBasics: coordinates
using CairoMakie
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


#######################################################################
#                                Plot                                 #
#######################################################################


GLMakie.closeall()


let

ylims = (; amax=(0.005,0.09))
yticks_cmon = -12:4:2
n = 11
plt_w, plt_h = 1200, 800
inset_xlims = (2800,3800)
inset_ylims = (; rsc=(0.003,0.025), amax=(0.01,0.038))

plotname    = joinpath(@__DIR__, "amax_infall_v_0.05_n_$n$(EXT)")
ssp_outputs     = find_runs(OUTPUTS["comparison4/naive_v_0.05"])
cts_ssp_outputs = find_runs(OUTPUTS["comparison4/cts_naive_v_0.05"])
cve_outputs     = find_runs(OUTPUTS["comparison4/helfer_v_0.05"])
cts_cve_outputs = find_runs(OUTPUTS["comparison4/cts_helfer_v_0.05"])

df_ssp     = load_runs(ssp_outputs)
df_cts_ssp = load_runs(cts_ssp_outputs)
df_cve     = load_runs(cve_outputs)
df_cts_cve = load_runs(cts_cve_outputs)

df_ssp.n     = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_ssp.output ]
df_cts_ssp.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_cts_ssp.output ]
df_cve.n     = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_cve.output ]
df_cts_cve.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_cts_cve.output ]

for df in (df_ssp, df_cts_ssp, df_cve, df_cts_cve)
    filter!(df) do row
        row.n == n
    end
end

fig = Figure(figure_padding=50)
gl = fig[1,1] = GridLayout()
ax_amax = Axis(gl[1,1], xlabel=L"t", ylabel=L"A_{\textrm{max}}(t)")
GWanalysis.get_output_0d(df_ssp,     "max/ana.AmpPsi")
GWanalysis.get_output_0d(df_cts_ssp, "max/ana.AmpPsi")
GWanalysis.get_output_0d(df_cve,     "max/ana.AmpPsi")
GWanalysis.get_output_0d(df_cts_cve, "max/ana.AmpPsi")
t_ssp        = df_ssp."t_max/ana.AmpPsi"[1]
t_cts_ssp    = df_cts_ssp."t_max/ana.AmpPsi"[1]
t_cve        = df_cve."t_max/ana.AmpPsi"[1]
t_cts_cve    = df_cts_cve."t_max/ana.AmpPsi"[1]
Amax_ssp     = df_ssp."max/ana.AmpPsi"[1]
Amax_cts_ssp = df_cts_ssp."max/ana.AmpPsi"[1]
Amax_cve     = df_cve."max/ana.AmpPsi"[1]
Amax_cts_cve = df_cts_cve."max/ana.AmpPsi"[1]

lines!(ax_amax, t_cve, Amax_cve,                          label="CVE")
lines!(ax_amax, t_cts_cve, Amax_cts_cve, linestyle=:dash, label="CTS+CVE")
lines!(ax_amax, t_ssp, Amax_ssp,                          label="SSP")
lines!(ax_amax, t_cts_ssp, Amax_cts_ssp, linestyle=:dash, label="CTS+SSP")
ax_amax.xticks[] = 0:100:650
ax_amax.yticks[] = 0.010:0.001:0.015
xlims!(ax_amax, (-25,575))
ylims!(ax_amax, (0.0108,0.0142))

# max_idx_ssp     = something(findmax(Amax_ssp))[2]
max_idx_cts_ssp = something(findmax(Amax_cts_ssp))[2]
# max_idx_cve     = something(findmax(Amax_cve))[2]
max_idx_cts_cve = something(findmax(Amax_cts_cve))[2]
# tmax_ssp     = t_ssp[max_idx_ssp]
tmax_cts_ssp = t_cts_ssp[max_idx_cts_ssp]
# tmax_cve     = t_cve[max_idx_cve]
tmax_cts_cve = t_cts_cve[max_idx_cts_cve]
# vlines!(ax_amax, tmax_ssp, color=:black, linestyle=:dash)
vlines!(ax_amax, tmax_cts_ssp, color=:black, linestyle=:dash, linewidth=3)
# vlines!(ax_amax, tmax_cve, color=:black, linestyle=:dash)
vlines!(ax_amax, tmax_cts_cve, color=:black, linestyle=:dot, linewidth=3)

Legend(gl[0,:], ax_amax, colgap=40,
       orientation=:horizontal, width=nothing, tellwidth=false)
rowgap!(gl, 0)

display(GLMakie.Screen(), fig)

@info "Saving plot to $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)


end
