using GWanalysis
using DataFrames
using GLMakie
import GLMakie.Makie.GeometryBasics: coordinates
using CairoMakie
using Printf
GLMakie.activate!()


include("../rootdirs.jl")
include("../theme.jl")


#######################################################################
#                         Plotting functions                          #
#######################################################################


function get_inset_bbox(inset, ax)
    shifted_bbox1 = lift(ax.scene.px_area, inset.layoutobservables.computedbbox,
                        ax.scene.camera.projectionview) do px_area, bbox, _
      Rect2f(bbox.origin .- px_area.origin, bbox.widths)
    end
end
function vertices_inset_and_bbox(inset, bbox, ax)
    vs = [ Observable{Any}() for _ = 1:4 ]
    onany(inset.finallimits, bbox) do bb1, bb2
      for (i,(c1, c2)) in enumerate(zip(coordinates(bb1), coordinates(bb2)))
        cc1 = Makie.project(ax.scene, Point2f(c1))
        vs[i][] = [ cc1, Point2f(c2) ]
      end
    end
    notify(inset.finallimits)
    return vs
end


trafo(v) = log10(abs(isinf(v) ? NaN : v))


function make_plot(df1, df2, label1, label2, ylims, yticks, inset_xlims, inset_ylims)

    fig = Figure()

    gl = fig[1,1] = GridLayout()

    ax_cmon = Axis(gl[1,1], ylabel=L"\log_{10}~|\mathcal{C}_{\textrm{mon}}|", xlabel=L"t")
    glinset = gl[1,1] = GridLayout(width=Relative(0.7), height=Relative(0.4),
                                        halign=0.8, valign=0.08, alignmode=Outside())
    # Box(glinset_rsc[1,1])
    inset  = Axis(glinset[1,1])
    translate!(inset.blockscene, 0, 0, 1000)
    hidedecorations!(inset, ticks=false, grid=false, minorgrid=false)

    GWanalysis.get_output_0d(df1, "integral/ana.Cmon")
    GWanalysis.get_output_0d(df1, "max/ana.AmpPsi")
    GWanalysis.get_output_0d(df2, "integral/ana.Cmon")
    GWanalysis.get_output_0d(df2, "max/ana.AmpPsi")

    t1, Cmon1 = df1."t_integral/ana.Cmon"[1], df1."integral/ana.Cmon"[1]
    t2, Cmon2 = df2."t_integral/ana.Cmon"[1], df2."integral/ana.Cmon"[1]
    Amax1 = df1."max/ana.AmpPsi"[1]
    Amax2 = df2."max/ana.AmpPsi"[1]

    max_idx2 = something(findmax(Amax2))[2]
    tmax2 = t2[max_idx2]
    idx1_lhs, idx1_rhs = findfirst(>=(inset_xlims[1]), t1), findfirst(>=(inset_xlims[2]), t1)
    idx2_lhs, idx2_rhs = findfirst(>=(inset_xlims[1]), t2), findfirst(>=(inset_xlims[2]), t2)

    lines!(ax_cmon, t1, trafo.(Cmon1), label=label1)
    lines!(ax_cmon, t2, trafo.(Cmon2), label=label2)

    lines!(inset, t1[idx1_lhs:idx1_rhs], trafo.(Cmon1[idx1_lhs:idx1_rhs]))
    lines!(inset, t2[idx2_lhs:idx2_rhs], trafo.(Cmon2[idx2_lhs:idx2_rhs]))

    vlines!(ax_cmon, tmax2, color=:black, linestyle=:dash)

    ylims!(ax_cmon, ylims)
    ax_cmon.yticks[] = yticks

    xlims!(inset, inset_xlims)
    ylims!(inset, inset_ylims)
    inset.yticks[] = yticks

    poly!(ax_cmon,  inset.finallimits,  color=(:white,0.0), strokecolor=:black, strokewidth=1)

    bbox  = get_inset_bbox(inset, ax_cmon)

    verts1 = vertices_inset_and_bbox(inset, bbox, ax_cmon)

    for vs in verts1
        l = lines!(ax_cmon, vs, color=(:grey,0.4), space=:pixel, linewidth=1)
        translate!(l, 0, 0, -100)
    end

    Legend(gl[0,:], ax_cmon, colgap=40,
           orientation=:horizontal, width=nothing, tellwidth=false)

    rowgap!(gl, 0)

    fig

end


#######################################################################
#                                Plot                                 #
#######################################################################


let

ylims_cmon = (-17, -1)
yticks_cmon = -16:4:2
n = 9
plt_w, plt_h = 1200, 500
inset_xlims = (1900,3200)
inset_ylims = (-9,-4)

out1    = find_runs(OUTPUTS["comparison4/cts_helfer_v_0.05"])
out2    = find_runs(OUTPUTS["comparison8/cts_helfer_v_0.05"])
plotname = joinpath(@__DIR__, "cmon_bdry_effect_cts+emf_v_0.05_n_11.cts_helfer$(EXT)")
# out1    = find_runs(OUTPUTS["comparison4/helfer_v_0.05"])
# out2    = find_runs(OUTPUTS["comparison8/helfer_v_0.05"])
# plotname = joinpath(@__DIR__, "cmon_bdry_effect_cts+emf_v_0.05_n_11.helfer$(EXT)")

df1 = load_runs(out1)
df2 = load_runs(out2)

df1.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df1.output ]
df2.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df2.output ]

filter!(df1) do row
    row.n == n
end
filter!(df2) do row
    row.n == n
end

fig = make_plot(df1, df2, L"R = 400", L"R = 800", ylims_cmon, yticks_cmon,
                inset_xlims, inset_ylims)
display(GLMakie.Screen(), fig)

@info "Saving plot to $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)


end
