using GWanalysis
using DataFrames
using GLMakie
import GLMakie.Makie.GeometryBasics: coordinates
using CairoMakie
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


#######################################################################
#                         Plotting functions                          #
#######################################################################


function get_inset_bbox(inset, ax)
    shifted_bbox1 = lift(ax.scene.px_area, inset.layoutobservables.computedbbox,
                        ax.scene.camera.projectionview) do px_area, bbox, _
      Rect2f(bbox.origin .- px_area.origin, bbox.widths)
    end
end
function vertices_inset_and_bbox(inset, bbox, ax)
    vs = [ Observable{Any}() for _ = 1:4 ]
    onany(inset.finallimits, bbox) do bb1, bb2
      for (i,(c1, c2)) in enumerate(zip(coordinates(bb1), coordinates(bb2)))
        cc1 = Makie.project(ax.scene, Point2f(c1))
        vs[i][] = [ cc1, Point2f(c2) ]
      end
    end
    notify(inset.finallimits)
    return vs
end


trafo(v) = log10(abs(isinf(v) ? NaN : v))


function make_plot(df1, df2, label1, label2, ylims, yticks, inset_xlims, inset_ylims)

    fig = Figure()

    gl = fig[1,1] = GridLayout()

    ax_cmon = Axis(gl[1,1])
    ax_rsc  = Axis(gl[2,1])
    ax_amax = Axis(gl[3,1], xlabel=L"t")
    glinset_rsc = gl[2,1] = GridLayout(width=Relative(0.7), height=Relative(0.5),
                                        halign=0.8, valign=0.9, alignmode=Outside())
    glinset_amaxs = gl[3,1] = GridLayout(width=Relative(0.7), height=Relative(0.5),
                                          halign=0.8, valign=0.9, alignmode=Outside())
    # Box(glinset_rsc[1,1])
    inset_rsc  = Axis(glinset_rsc[1,1])
    inset_amax = Axis(glinset_amaxs[1,1])
    translate!(inset_rsc.blockscene, 0, 0, 1000)
    translate!(inset_amax.blockscene, 0, 0, 1000)

    GWanalysis.get_output_0d(df1, "integral/ana.Cmon")
    GWanalysis.get_output_0d(df1, "origin/ana.Rsc")
    GWanalysis.get_output_0d(df1, "max/ana.AmpPsi")
    GWanalysis.get_output_0d(df2, "integral/ana.Cmon")
    GWanalysis.get_output_0d(df2, "origin/ana.Rsc")
    GWanalysis.get_output_0d(df2, "max/ana.AmpPsi")

    t1, Cmon1 = df1."t_integral/ana.Cmon"[1], df1."integral/ana.Cmon"[1]
    t2, Cmon2 = df2."t_integral/ana.Cmon"[1], df2."integral/ana.Cmon"[1]
    Rsc1 = df1."origin/ana.Rsc"[1]
    Rsc2 = df2."origin/ana.Rsc"[1]
    Amax1 = df1."max/ana.AmpPsi"[1]
    Amax2 = df2."max/ana.AmpPsi"[1]

    max_idx2 = something(findmax(Amax2))[2]
    tmax2 = t2[max_idx2]
    idx1_lhs, idx1_rhs = findfirst(>=(inset_xlims[1]), t1), findfirst(>=(inset_xlims[2]), t1)
    idx2_lhs, idx2_rhs = findfirst(>=(inset_xlims[1]), t2), findfirst(>=(inset_xlims[2]), t2)

    lines!(ax_cmon, t1, trafo.(Cmon1), label=label1)
    lines!(ax_cmon, t2, trafo.(Cmon2), label=label2, linestyle=:dash)
    lines!(ax_rsc, t1, Rsc1)
    lines!(ax_rsc, t2, Rsc2, linestyle=:dash)
    lines!(ax_amax, t1, Amax1)
    lines!(ax_amax, t2, Amax2, linestyle=:dash)

    lines!(inset_rsc, t1[idx1_lhs:idx1_rhs], Rsc1[idx1_lhs:idx1_rhs])
    lines!(inset_rsc, t2[idx2_lhs:idx2_rhs], Rsc2[idx2_lhs:idx2_rhs], linestyle=:dash)
    lines!(inset_amax, t1[idx1_lhs:idx1_rhs], Amax1[idx1_lhs:idx1_rhs])
    lines!(inset_amax, t2[idx2_lhs:idx2_rhs], Amax2[idx2_lhs:idx2_rhs], linestyle=:dash)

    vlines!(ax_cmon, tmax2, color=:black, linestyle=:dash)
    vlines!(ax_rsc,  tmax2, color=:black, linestyle=:dash)
    vlines!(ax_amax, tmax2, color=:black, linestyle=:dash)

    ylims!(ax_cmon, ylims.cmon)
    ax_cmon.yticks[] = yticks
    ylims!(ax_rsc, ylims.rsc)
    ylims!(ax_amax, ylims.amax)

    ylims!(inset_rsc, inset_ylims.rsc)
    xlims!(inset_rsc, inset_xlims)
    xlims!(inset_amax, inset_xlims)
    ylims!(inset_amax, inset_ylims.amax)
    inset_rsc.yticks[]  = WilkinsonTicks(2)
    inset_amax.yticks[] = WilkinsonTicks(2)

    reset_limits!(inset_rsc, xauto=false, yauto=false)
    reset_limits!(inset_amax, xauto=false, yauto=false)

    poly!(ax_rsc,  inset_rsc.finallimits,  color=(:white,0.0), strokecolor=:black, strokewidth=1)
    poly!(ax_amax, inset_amax.finallimits, color=(:white,0.0), strokecolor=:black, strokewidth=1)

    bbox_rsc  = get_inset_bbox(inset_rsc, ax_rsc)
    bbox_amax = get_inset_bbox(inset_amax, ax_amax)

    verts1 = vertices_inset_and_bbox(inset_rsc, bbox_rsc, ax_rsc)
    verts2 = vertices_inset_and_bbox(inset_amax, bbox_amax, ax_amax)

    for (ax,verts) in zip((ax_rsc,ax_amax),(verts1,verts2))
        for vs in verts
            l = lines!(ax, vs, color=(:grey,0.4), space=:pixel, linewidth=1)
            translate!(l, 0, 0, -100)
        end
    end

    # hide axis decorations
    hidexdecorations!(ax_cmon, ticks=false, grid=false, minorgrid=false)
    hidexdecorations!(ax_rsc, ticks=false, grid=false, minorgrid=false)

    linkxaxes!(ax_cmon, ax_rsc, ax_amax)

    Legend(gl[0,:], ax_cmon, colgap=40,
           orientation=:horizontal, width=nothing, tellwidth=false)

    Label(gl[1,0], text=L"\log_{10}~|\mathcal{C}_{\textrm{mon}}|", rotation=pi/2, tellheight=false)
    Label(gl[2,0], text=L"R(t,x=0)", rotation=pi/2, tellheight=false)
    Label(gl[3,0], text=L"A_{\textrm{max}}", rotation=pi/2, tellheight=false)

    rowgap!(gl, 0)

    fig

end


#######################################################################
#                                Plot                                 #
#######################################################################


GLMakie.closeall()


let

ylims = (; cmon=(-13,-1), rsc=(-0.005,0.07), amax=(0.005,0.09))
yticks_cmon = -12:4:2
n = 11
plt_w, plt_h = 1200, 1200
inset_xlims = (2800,3800)
inset_ylims = (; rsc=(0.003,0.025), amax=(0.01,0.038))

plotname = joinpath(@__DIR__, "cmon_rsc_amax_cve_vs_cts+cve_v_0.05_n_$n$(EXT)")

cve_outputs = find_runs(OUTPUTS["comparison4/helfer_v_0.05"])
cts_outputs = find_runs(OUTPUTS["comparison4/cts_helfer_v_0.05"])

df_cve = load_runs(cve_outputs)
df_cts = load_runs(cts_outputs)

df_cve.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_cve.output ]
df_cts.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_cts.output ]

filter!(df_cve) do row
    row.n == n
end
filter!(df_cts) do row
    row.n == n
end

fig = make_plot(df_cts, df_cve, "CTS+CVE", "CVE", ylims, yticks_cmon, inset_xlims, inset_ylims)
display(GLMakie.Screen(), fig)

@info "Saving plot to $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)


end


let

ylims = (; cmon=(-13,-1), rsc=(-0.005,0.07), amax=(0.005,0.09))
yticks_cmon = -12:4:2
n = 11
plt_w, plt_h = 1200, 1200
inset_xlims = (2800,3800)
inset_ylims = (; rsc=(0.003,0.025), amax=(0.01,0.038))

plotname    = joinpath(@__DIR__, "cmon_rsc_amax_ssp_vs_cts+ssp_v_0.05_n_$n$(EXT)")
ssp_outputs = find_runs(OUTPUTS["comparison4/naive_v_0.05"])
cts_outputs = find_runs(OUTPUTS["comparison4/cts_naive_v_0.05"])

df_ssp = load_runs(ssp_outputs)
df_cts = load_runs(cts_outputs)

df_ssp.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_ssp.output ]
df_cts.n = [ GWanalysis.getparameter(Int64, out, "grid.n.xyz") for out in df_cts.output ]

filter!(df_ssp) do row
    row.n == n
end
filter!(df_cts) do row
    row.n == n
end

fig = make_plot(df_cts, df_ssp, "CTS+SSP", "SSP", ylims, yticks_cmon, inset_xlims, inset_ylims)
display(GLMakie.Screen(), fig)

@info "Saving plot to $plotname"
save(plotname, fig, resolution=(plt_w,plt_h), backend=CairoMakie)


end
