using GWanalysis
using DataFrames
using GLMakie
using CairoMakie
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


#######################################################################
#                               Script                                #
#######################################################################

let


M = 0.79
Nextrp = 3
boosts = [ "0.0", "0.05", "0.1", "0.15", "0.2" ]
str_fcut = "fcut2"
rext = 720

plotname = "dEdt_comparison_wrt_v_$(str_fcut)_Nextrp_$Nextrp$(EXT)"

data = Any[]

for (i,b) in enumerate(boosts), Ne in Nextrp

    outputs = find_runs(OUTPUTS["comparison8/cts_helfer_v_$b"])
    filter!(outputs) do out
        endswith(out, "p15")
    end
    df = load_runs(outputs)

    N_Psi4, u_Psi4, Psi4s = GWanalysis.get_output_analysis(df, "win_Psi4_20_$str_fcut")
    N_dhdt, u_dhdt, dhdts = GWanalysis.get_output_analysis(df, "dhdt_Psi4_20_$str_fcut")
    N_h, u_h, hs = GWanalysis.get_output_analysis(df, "h_Psi4_20_$str_fcut")
    N_E, u_E, Es = GWanalysis.get_output_analysis(df, "E_Psi4_20_$str_fcut")
    t_Rsc, Rsc = GWanalysis.get_output_0d(df, "origin/ana.Rsc")
    t_Amp, Amp = GWanalysis.get_output_0d(df, "max/ana.AmpPsi")
    t_N, N = GWanalysis.get_output_0d(df, "integral/ana.noetherQintegrand")
    t_N, N = GWanalysis.get_output_0d(df, "integral/ana.noetherQintegrand")
    vec_rs, vec_ts_Madms, vec_Madms = GWanalysis.get_output_spheres2(df, "ana.admMintegrand")

    idx = findfirst(N -> N == Ne, N_Psi4[1]) |> something
    Psi4 = Psi4s[1][idx]
    dhdt = dhdts[1][idx]
    h = hs[1][idx]
    E = Es[1][idx]
    u_Psi4 = u_Psi4[1]
    u_dhdt = u_dhdt[1]
    u_h = u_h[1]
    u_E = u_E[1]
    t_Rsc = t_Rsc[1]
    Rsc = Rsc[1]
    t_Amp = t_Amp[1]
    Amp = Amp[1]
    t_N = t_N[1]
    N = N[1]
    dNdt = [ (N[i+1]-N[i])/(t_N[i+1]-t_N[i]) for i in eachindex(t_N,N)[1:end-1] ]

    dt = diff(t_N)
    @assert all(dt .> 0.0)

    idx_r = findfirst(r -> r == rext, vec_rs[1]) |> something
    t_Madm = vec_ts_Madms[1]
    Madm = vec_Madms[1][idx_r]

    _, maxidx = findmax(Amp)
    t_max = t_Amp[maxidx]

    key = "meta_dhdt_Psi4_20_$str_fcut"
    meta = getproperty(df, key) |> first
    # str_fcut = meta["Filter"]["fcut"]
    fcut = tryparse(Float64,meta["Filter"]["fcut"])
    umin = tryparse(Float64, meta["Window"]["umin"])
    umax = tryparse(Float64, meta["Window"]["umax"])

    push!(data, (; Psi4, dhdt, h, E, u_Psi4, u_dhdt, u_h, u_E, meta, fcut, umin, umax, boost=b,
                   t_Rsc, Rsc, t_Amp, Amp, t_max, t_N, N, t_Madm, Madm, dNdt))
end



fig = Figure()
g = fig[1,1] = GridLayout()

# ax_dEdt = Axis(g[1,1], xlabel=L"u/M_{\textrm{BBS}}", ylabel=L"dE/dt")
# for d in data
#     u = d.u_dhdt
#     dEdt = @. d.dhdt^2 / (16*pi)
#     lines!(ax_dEdt, u, dEdt, label=L"v = %$(d.boost)")
# end
# Legend(fig[0,1], ax_dEdt, orientation=:horizontal)

ax_E = Axis(g[1,1], xlabel=L"u/M_{\textrm{BBS}}")#, ylabel=L"E/M_{\textrm{BBS}}~[%]")
for (i,d) in enumerate(data)
    u = d.u_E
    E = d.E
    ls = isodd(i) ? :solid : :dash
    lines!(ax_E, u./M, E./M.*100, label=L"v = %$(d.boost)", linestyle=ls)
    vlines!(ax_E, d.t_max/M, linestyle=:dash, color=:black)
end
Label(g[1,0], text=L"E(u)/M_{\textrm{BBS}}~[%]", tellheight=false, rotation=pi/2)

ax_Rsc = Axis(g[2,1], xlabel=L"t/M_{\textrm{BBS}}")#, ylabel=L"R(t,x=0)")
for (i,d) in enumerate(data)
    ls = isodd(i) ? :solid : :dash
    lines!(ax_Rsc, d.t_Rsc./M, d.Rsc, label=L"v = %$(d.boost)", linestyle=ls)
    vlines!(ax_Rsc, d.t_max/M, linestyle=:dash, color=:black)
end
Label(g[2,0], text=L"R(t,x=0)", tellheight=false, rotation=pi/2)

ax_Amp = Axis(g[3,1], xlabel=L"t/M_{\textrm{BBS}}")#, ylabel=L"A_{\textrm{max}}(t)")
for (i,d) in enumerate(data)
    ls = isodd(i) ? :solid : :dash
    lines!(ax_Amp, d.t_Amp./M, d.Amp, label=L"v = %$(d.boost)", linestyle=ls)
    vlines!(ax_Amp, d.t_max/M, linestyle=:dash, color=:black)
end
Label(g[3,0], text=L"A_{\textrm{max}}(t)", tellheight=false, rotation=pi/2)

ax_N = Axis(g[4,1], xlabel=L"t/M_{\textrm{BBS}}")#, ylabel=L"A_{\textrm{max}}(t)")
for (i,d) in enumerate(data)
    ls = isodd(i) ? :solid : :dash
    lines!(ax_N, d.t_N./M, d.N, label=L"v = %$(d.boost)", linestyle=ls)
    vlines!(ax_N, d.t_max/M, linestyle=:dash, color=:black)
end
Label(g[4,0], text=L"N(t)", tellheight=false, rotation=pi/2)

# ax_dNdt = Axis(g[5,1], xlabel=L"t/M_{\textrm{BBS}}")#, ylabel=L"A_{\textrm{max}}(t)")
# for (i,d) in enumerate(data)
#     ls = isodd(i) ? :solid : :dash
#     lines!(ax_dNdt, d.t_N[1:end-1]./M, d.dNdt, label=L"v = %$(d.boost)", linestyle=ls)
#     vlines!(ax_dNdt, d.t_max/M, linestyle=:dash, color=:black)
# end
# Label(g[5,0], text=L"dN/dt(t)", tellheight=false, rotation=pi/2)

ax_Madm = Axis(g[5,1], xlabel=L"t/M_{\textrm{BBS}}")#, ylabel=L"A_{\textrm{max}}(t)")
for (i,d) in enumerate(data)
    ls = isodd(i) ? :solid : :dash
    lines!(ax_Madm, d.t_Madm./M, d.Madm, label=L"v = %$(d.boost)", linestyle=ls)
    vlines!(ax_Madm, d.t_max/M, linestyle=:dash, color=:black)
end
Label(g[5,0], text=L"M_{\textrm{ADM}}(t)", tellheight=false, rotation=pi/2)

hidexdecorations!(ax_Rsc, ticks=false, minorticks=false, grid=false)
hidexdecorations!(ax_Amp, ticks=false, minorticks=false, grid=false)
hidexdecorations!(ax_N, ticks=false, minorticks=false, grid=false)
# hidexdecorations!(ax_dNdt, ticks=false, minorticks=false, grid=false)
# linkxaxes!(ax_Madm, ax_dNdt, ax_N, ax_Amp, ax_Rsc, ax_E)
linkxaxes!(ax_Madm, ax_N, ax_Amp, ax_Rsc, ax_E)
Legend(g[0,1], ax_E, width=nothing, tellheight=true, orientation=:horizontal,
       colgap=20, patchsize=(40,20))#, labelsize=30)
rowgap!(g, 1, 0)
rowgap!(g, 3, 0)
rowgap!(g, 4, 0)
rowgap!(g, 5, 0)
# rowgap!(g, 6, 0)

xlims!(ax_E, (-100,4200))
xlims!(ax_Rsc, (-100,4200))
xlims!(ax_Amp, (-100,4200))
xlims!(ax_N, (-100,4200))
# xlims!(ax_dNdt, (-100,4200))
xlims!(ax_Madm, (-100,4200))

ylims!(ax_E, (-0.002,0.027))
ylims!(ax_Rsc, (-0.01, 0.07))
ylims!(ax_Amp, (-0.01, 0.07))
ylims!(ax_N, (0.68, 0.83))
# ylims!(ax_dNdt, (-9e-5, 3e-5))
ylims!(ax_Madm, (0.65, 0.83))

ax_Rsc.yticks[]  = -1.0:0.02:1.0
ax_Amp.yticks[]  = -1.0:0.02:1.0
ax_N.yticks[]    = 0.5:0.04:1.0
ax_Madm.yticks[] = 0.5:0.04:1.0
# ylims!(ax_N, (0.71,0.83))


filename = joinpath(@__DIR__, plotname)
@info "saving plot to $filename..."
save(filename, fig; resolution=(1200,2000), backend=CairoMakie)


display(fig)


end
