using GWanalysis
using DataFrames
using GLMakie
using CairoMakie
import GLMakie.Makie.GeometryBasics: coordinates
using Printf
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


function get_inset_bbox(inset, ax)
    shifted_bbox1 = lift(ax.scene.px_area, inset.layoutobservables.computedbbox,
                        ax.scene.camera.projectionview) do px_area, bbox, _
      Rect2f(bbox.origin .- px_area.origin, bbox.widths)
    end
end
function vertices_inset_and_bbox(inset, bbox, ax)
    vs = [ Observable{Any}() for _ = 1:4 ]
    onany(inset.finallimits, bbox) do bb1, bb2
      for (i,(c1, c2)) in enumerate(zip(coordinates(bb1), coordinates(bb2)))
        cc1 = Makie.project(ax.scene, Point2f(c1))
        vs[i][] = [ cc1, Point2f(c2) ]
      end
    end
    notify(inset.finallimits)
    return vs
end


let

dirs = find_runs(OUTPUTS["longruns/cts_helfer_v_0.05"])
df = load_runs(dirs)

plotname = "cmon_psi4_longrun$(EXT)"

radius = 720 # from which we take Psi4
M = 2 * 0.395

plt_w, plt_h = 2400, 800

df.n = [ GWanalysis.getparameter(Float64, out, "grid.n.xyz") for out in df.output ]
df = filter(df) do row
    row.n <= 17
end
sort!(df, :n)

GWanalysis.get_output_0d(df, "max/ana.AmpPsi")
GWanalysis.get_output_0d(df, "integral/ana.Cmon")
GWanalysis.get_output_spheres_modes(df, "Psi4_20")

ts_AmpPsi = df."t_max/ana.AmpPsi"
AmpPsis = df."max/ana.AmpPsi"
ts_Cmon = df."t_integral/ana.Cmon"
Cmons = df."integral/ana.Cmon"
vec_rs = df."r_Psi4_20"
ts_Re = df."t_Psi4_20"
vec_Res = df."RePsi4_20"

_, idx_max = findmax(AmpPsis[end])
t_max = ts_AmpPsi[end][idx_max]

idxs  = [ findfirst(ri -> ri == radius, rs) |> something for rs in vec_rs ]
display(idxs)
Res   = [ Res[i] for (Res,i) in zip(vec_Res,idxs) ]

# Amax = df."max/ana.AmpPsi"[1]
# _, idx_max = findmax(Amax) |> something
# tmax = ts_Cmon[1][idx_max]
# @info tmax

fig = Figure()
ga = fig[1,1] = GridLayout()

ax_cmon = Axis(ga[1,1])
ax_psi4 = Axis(ga[2,1], xlabel=L"t/M_{\textrm{BBS}}")

Label(ga[1,0], text=L"\log_{10}|\mathcal{C}_{\textrm{mon}}|",
      rotation=pi/2, tellheight=false)
Label(ga[2,0], text=L"M_{\textrm{BBS}}~r_{\textrm{ext}}~\Psi_{4,20}",
      rotation=pi/2, tellheight=false)

# inset_gl = ga[3,1] = GridLayout(ga[3,1]; width=Relative(0.76), height=Relative(0.65),
#                                 halign=0.9, valign=0.5)
# inset1_psi4 = Axis(inset_gl[1,1])
# inset2_psi4 = Axis(inset_gl[1,2])
# translate!(inset1_psi4.blockscene, 0, 0, 1000)
# translate!(inset2_psi4.blockscene, 0, 0, 1000)
# hideydecorations!(inset2_psi4, ticks=false, grid=false, minorgrid=false, minorticks=false)
# colgap!(inset_gl, 0)

trafo = (v -> isinf(v) ? NaN : v) ∘ log10 ∘ abs

for i = 1:nrow(df)
    t_Cmon, Cmon = ts_Cmon[i], Cmons[i]
    n = Int(df.n[i])
    c = Makie.wong_colors()[i]
    cc = RGBAf(c.r, c.g, c.b, i%2 == nrow(df)%2 ? 1.0 : 0.6)
    lines!(ax_cmon, t_Cmon./M, trafo.(Cmon), label=L"n = %$n", color=cc)
end
vlines!(ax_cmon, t_max./M, color=:black, linestyle=:dash)
# only plot Psi4 for highest resolved run
t_Re, Re = ts_Re[end], Res[end]
n = Int(df.n[end])
# u_max = GWanalysis.retarded_Schwarzschild_time(t_max, radius, M)
lines!(ax_psi4, t_Re./M, Re, color=Makie.wong_colors()[nrow(df)])
vlines!(ax_psi4, t_max./M, color=:black, linestyle=:dash)
# xlims1 = (9000, 9500)
# xlims2 = (19000, 19500)
# i1l, i1r = findfirst(>=(xlims1[1]), t_Re) |> something, findfirst(>=(xlims1[2]), t_Re) |> something
# lines!(inset1_psi4, t_Re[i1l:i1r], M .* Re[i1l:i1r])
# i2l, i2r = findfirst(>=(xlims2[1]), t_Re) |> something, findfirst(>=(xlims2[2]), t_Re) |> something
# lines!(inset2_psi4, t_Re[i2l:i2r], M .* Re[i2l:i2r])

# linkyaxes!(inset1_psi4, inset2_psi4)
# xlims!(inset1_psi4, xlims1)
# xlims!(inset2_psi4, xlims2)
#
# poly!(ax_psi4, inset1_psi4.finallimits, color=(:white,0.0), strokecolor=:black, strokewidth=1)
# poly!(ax_psi4, inset2_psi4.finallimits, color=(:white,0.0), strokecolor=:black, strokewidth=1)
#
# bbox1 = get_inset_bbox(inset1_psi4, ax_psi4)
# bbox2 = get_inset_bbox(inset2_psi4, ax_psi4)
#
# verts1 = vertices_inset_and_bbox(inset1_psi4, bbox1, ax_psi4)
# verts2 = vertices_inset_and_bbox(inset2_psi4, bbox2, ax_psi4)
#
# for verts in (verts1,verts2)
#     for vs in verts
#         lines!(ax_psi4, vs, color=:black, space=:pixel, linewidth=1)
#     end
# end

hidexdecorations!(ax_cmon, ticks=false, grid=false, minorgrid=false, minorticks=false)
ax_psi4.xtickformat[] = xs -> [ @sprintf "%.0f" x for x in xs ]
ax_cmon.xticks[] = 0:2500:25000
ax_psi4.xticks[] = 0:2500:25000
Legend(ga[0,1], ax_cmon, orientation=:horizontal, colgap=30,
       width=nothing, tellwidth=false)
rowgap!(ga, 0)

display(fig)


plotname = joinpath(@__DIR__, plotname)
@info "saving '$plotname'"
save(plotname, fig, resolution=(plt_w, plt_h), backend=CairoMakie)


end
