using GWanalysis
using DelimitedFiles
using DataFrames
using Printf
using FFTW


include("../rootdirs.jl")


#######################################################################
#                              Functions                              #
#######################################################################


function align_by_zeros(u_window, n_zeros, rs, u_intrp, Res_intrp)

    u_winmin, u_winmax = u_window
    i_min = findfirst(ui -> ui > u_winmin, u_intrp) |> something
    i_max = findfirst(ui -> ui > u_winmax, u_intrp) |> something
    u_win = u_intrp[i_min:i_max]

    Nrs = length(rs)
    i_zeros = [ zeros(Int64, n_zeros) for _ = 1:Nrs ]
    for n = 1:Nrs
        Re = Res_intrp[n]
        j_zero = 1
        for k = 1:length(u_win)
            if Re[k+i_min-1] * Re[k+i_min] < 0
                i_zeros[n][j_zero] = k+i_min-1
                j_zero += 1
                j_zero > n_zeros && break
            end
        end
    end

    u_zeros_lhs   = [ [ u_intrp[i]   for i in i_zero ] for i_zero in i_zeros ]
    u_zeros_rhs   = [ [ u_intrp[i+1] for i in i_zero ] for i_zero in i_zeros ]
    u_zeros_intrp = [ zeros(Float64, n_zeros) for _ = 1:Nrs ]
    for n = 1:Nrs
        Re = Res_intrp[n]
        is = i_zeros[n]
        for (ii,i) in enumerate(is)
            ul, ur = u_intrp[i], u_intrp[i+1]
            Rel, Rer = Re[i], Re[i+1]
            u = - Rel * (ur - ul) / (Rer - Rel) + ul
            u_zeros_intrp[n][ii] = u
        end
    end

    du_mean  = [ sum(u_zeros) / n_zeros for u_zeros in u_zeros_intrp ]
    diff_dus = [ du_mean[1]-du_mean[1+i] for i = 1:Nrs-1 ]

    return diff_dus

    # us_shifted = [ u .+ d for (u,d) in zip(us[2:end],diff_dus) ]
    # pushfirst!(us_shifted, us[1])
    # Res_intrp_shifted, u_intrp_shifted = GWanalysis.interpolate_common_grid(us_shifted, Res)
    #
    # return us_shifted, Res_intrp_shifted
end



function psi4_extrapolate(outs, u_window, n_zeros, Ns_extrp, psi4_mode, M)

    df = load_runs(outs)

    all_rs, all_t, all_Res, all_Ims, all_us = GWanalysis.get_output_spheres_modes(df, psi4_mode, M)

    for (i,row) in enumerate(eachrow(df))
        rs  = all_rs[i]
        t   = all_t[i]
        Res = all_Res[i]
        us  = all_us[i]


        Res_intrp, u_intrp = GWanalysis.interpolate_common_grid(us, Res)

        diff_dus = []
        try
            diff_dus = align_by_zeros(u_window, n_zeros, rs, u_intrp, Res_intrp)
        catch e
            @warn "alignment by zeros failed, skipping ..."
            continue
        end
        us_shifted = [ u .+ d for (u,d) in zip(us[2:end],diff_dus) ]
        pushfirst!(us_shifted, us[1])
        Res_intrp_shifted, u_intrp_shifted = GWanalysis.interpolate_common_grid(us_shifted, Res)

        # du = u_intrp_shifted[2] - u_intrp_shifted[1]
        # fs = rfftfreq(length(u_intrp_shifted), 1/du)
        # rfft_Res = [ rfft(Re) for Re in Res_intrp_shifted ]

        # amp_rfft_Res = [ abs.(ft) for ft in rfft_Res ]
        # pha_rfft_Res = [ angle.(ft) for ft in rfft_Res ]

        # GWanalysis.collective_unwrap!(pha_rfft_Res)
        # rfft_Res_recons = [ amp .* exp.(im*phi) for (amp,phi) in zip(amp_rfft_Res, pha_rfft_Res) ]
        # Res_recons = irfft.(rfft_Res_recons, length(u_intrp_shifted))

        Res_extrps_shifted = [ GWanalysis.extrapolate_psi4(u_intrp_shifted, Res_intrp_shifted, rs; N_extrp=N)
                              for N in Ns_extrp ]

        rfft_Res_extrp = [ rfft(Re_coeffs[1]) for Re_coeffs in Res_extrps_shifted ]
        # amp_rfft_Res_extrp = [ abs.(ft) for ft in rfft_Res_extrp ]
        # pha_rfft_Res_extrp = [ angle.(ft) for ft in rfft_Res_extrp ]

        analysis_dir = joinpath(row.output.dir, "analysis")
        !isdir(analysis_dir) && mkdir(analysis_dir)

        filename = joinpath(analysis_dir, "$(psi4_mode)_intrp")
        @info "writing retarded and time aligned data to $filename"
        open(filename, "w") do file
            print(file, "# Time")
            for r in rs
                @printf(file, " r=%.1f", r)
            end
            println(file)
            data = hcat( u_intrp, Res_intrp... )
            writedlm(file, data)
        end

        for (i,N) in enumerate(Ns_extrp)
            filename = joinpath(analysis_dir, "$(psi4_mode)_Nextrp_$N")
            @info "writing extrapolated data to $filename"
            open(filename, "w") do file
                println(file, "# Header: Time, Psi4")
                data = hcat( u_intrp_shifted, Res_extrps_shifted[i][1] )
                writedlm(file, data)
            end
        end

    end

end


#######################################################################
#                               Script                                #
#######################################################################


let

n_zeros = 1
Ns_extrp = [ 1, 2, 3, 4, 5 ]
psi4_mode = "Psi4_20"
M = 0.79402


outs = [ find_runs(OUTPUTS["comparison8/cts_helfer_v_0.0"]),
         find_runs(OUTPUTS["comparison8/cts_naive_v_0.0"]),
         find_runs(OUTPUTS["comparison8/helfer_v_0.0"]),
         find_runs(OUTPUTS["comparison8/naive_v_0.0"]) ]
u_window = (800, 900)
for out in outs
    psi4_extrapolate(out, u_window, n_zeros, Ns_extrp, psi4_mode, M)
end

outs = [ find_runs(OUTPUTS["comparison8/cts_helfer_v_0.05"]),
         find_runs(OUTPUTS["comparison8/cts_naive_v_0.05"]),
         find_runs(OUTPUTS["comparison8/helfer_v_0.05"]),
         find_runs(OUTPUTS["comparison8/naive_v_0.05"]) ]
u_window = (400, 500)
for out in outs
    psi4_extrapolate(out, u_window, n_zeros, Ns_extrp, psi4_mode, M)
end

outs = [ find_runs(OUTPUTS["comparison8/cts_helfer_v_0.1"]),
         find_runs(OUTPUTS["comparison8/cts_naive_v_0.1"]),
         find_runs(OUTPUTS["comparison8/helfer_v_0.1"]),
         find_runs(OUTPUTS["comparison8/naive_v_0.1"]) ]
u_window = (250, 350)
for out in outs
    psi4_extrapolate(out, u_window, n_zeros, Ns_extrp, psi4_mode, M)
end

outs = [ find_runs(OUTPUTS["comparison8/cts_helfer_v_0.15"]),
         find_runs(OUTPUTS["comparison8/cts_naive_v_0.15"]),
         find_runs(OUTPUTS["comparison8/helfer_v_0.15"]),
         find_runs(OUTPUTS["comparison8/naive_v_0.15"]) ]
u_window = (200, 300)
for out in outs
    psi4_extrapolate(out, u_window, n_zeros, Ns_extrp, psi4_mode, M)
end


outs = [ find_runs(OUTPUTS["comparison8/cts_helfer_v_0.2"]),
         find_runs(OUTPUTS["comparison8/cts_naive_v_0.2"]),
         find_runs(OUTPUTS["comparison8/helfer_v_0.2"]),
         find_runs(OUTPUTS["comparison8/naive_v_0.2"]) ]
u_window = (150, 250)
for out in outs
    psi4_extrapolate(out, u_window, n_zeros, Ns_extrp, psi4_mode, M)
end

end
