using GWanalysis
using DataFrames
using FFTW
using GLMakie
using Printf
using CairoMakie
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")


trafo(v) = (isinf(v) || v == 0.0) ? NaN : (log10 ∘ abs)(v)


#######################################################################
#                               Script                                #
#######################################################################

let


M = 0.79
Nextrp = 1:5
boosts = [ "0.0", "0.05", "0.1", "0.15", "0.2" ]

for b in boosts, Ne in Nextrp

    plotname = joinpath(@__DIR__, "psi_dhdt_h_comparison8_vary_fcuts_v_$(b)_Nextrp_$Ne$(EXT)")
    outputs = find_runs(OUTPUTS["comparison8/cts_helfer_v_$b"])
    filter!(outputs) do out
        endswith(out, "p15")
    end
    df = load_runs(outputs)

    # dhdt1 = "dhdt_Psi4_20_fcut1"
    # dhdt2 = "dhdt_Psi4_20_fcut1"
    # dhdt3 = "dhdt_Psi4_20_fcut1"
    #
    # h1 = "h_Psi4_20_fcut1"
    # h2 = "h_Psi4_20_fcut1"
    # h3 = "h_Psi4_20_fcut1"

    N_Psi41, u_Psi41, Psi4s1 = GWanalysis.get_output_analysis(df, "win_Psi4_20_fcut1")
    N_Psi42, u_Psi42, Psi4s2 = GWanalysis.get_output_analysis(df, "win_Psi4_20_fcut2")
    N_Psi43, u_Psi43, Psi4s3 = GWanalysis.get_output_analysis(df, "win_Psi4_20_fcut3")
    N_dhdt1, u_dhdt1, dhdts1 = GWanalysis.get_output_analysis(df, "dhdt_Psi4_20_fcut1")
    N_dhdt2, u_dhdt2, dhdts2 = GWanalysis.get_output_analysis(df, "dhdt_Psi4_20_fcut2")
    N_dhdt3, u_dhdt3, dhdts3 = GWanalysis.get_output_analysis(df, "dhdt_Psi4_20_fcut3")
    N_h1, u_h1, hs1 = GWanalysis.get_output_analysis(df, "h_Psi4_20_fcut1")
    N_h2, u_h2, hs2 = GWanalysis.get_output_analysis(df, "h_Psi4_20_fcut2")
    N_h3, u_h3, hs3 = GWanalysis.get_output_analysis(df, "h_Psi4_20_fcut3")
    _, _, nobw_dhdts1 = GWanalysis.get_output_analysis(df, "no_butterworth_dhdt_Psi4_20_fcut1")
    _, _, nobw_dhdts2 = GWanalysis.get_output_analysis(df, "no_butterworth_dhdt_Psi4_20_fcut2")
    _, _, nobw_dhdts3 = GWanalysis.get_output_analysis(df, "no_butterworth_dhdt_Psi4_20_fcut3")
    _, _, nobw_hs1 = GWanalysis.get_output_analysis(df, "no_butterworth_h_Psi4_20_fcut1")
    _, _, nobw_hs2 = GWanalysis.get_output_analysis(df, "no_butterworth_h_Psi4_20_fcut2")
    _, _, nobw_hs3 = GWanalysis.get_output_analysis(df, "no_butterworth_h_Psi4_20_fcut3")


    idx1 = findfirst(N -> N == Ne, N_Psi41[1]) |> something
    idx2 = findfirst(N -> N == Ne, N_Psi42[1]) |> something
    idx3 = findfirst(N -> N == Ne, N_Psi43[1]) |> something
    Psi41 = Psi4s1[1][idx1]
    Psi42 = Psi4s2[1][idx2]
    Psi43 = Psi4s3[1][idx3]
    dhdt1 = dhdts1[1][idx1]
    dhdt2 = dhdts2[1][idx2]
    dhdt3 = dhdts3[1][idx3]
    h1 = hs1[1][idx1]
    h2 = hs2[1][idx2]
    h3 = hs3[1][idx3]
    nobw_dhdt1 = nobw_dhdts1[1][idx1]
    nobw_dhdt2 = nobw_dhdts2[1][idx2]
    nobw_dhdt3 = nobw_dhdts3[1][idx3]
    nobw_h1 = nobw_hs1[1][idx1]
    nobw_h2 = nobw_hs2[1][idx2]
    nobw_h3 = nobw_hs3[1][idx3]
    u_Psi41 = u_Psi41[1]
    u_Psi42 = u_Psi42[1]
    u_Psi43 = u_Psi43[1]
    u_dhdt1 = u_dhdt1[1]
    u_dhdt2 = u_dhdt2[1]
    u_dhdt3 = u_dhdt3[1]
    u_h1 = u_h1[1]
    u_h2 = u_h2[1]
    u_h3 = u_h3[1]


    meta1 = df."meta_dhdt_Psi4_20_fcut1" |> first
    meta2 = df."meta_dhdt_Psi4_20_fcut2" |> first
    meta3 = df."meta_dhdt_Psi4_20_fcut3" |> first
    str_fcut1 = meta1["Filter"]["fcut"]
    str_fcut2 = meta2["Filter"]["fcut"]
    str_fcut3 = meta3["Filter"]["fcut"]
    fcut1 = eval(Meta.parse(str_fcut1))
    fcut2 = eval(Meta.parse(str_fcut2))
    fcut3 = eval(Meta.parse(str_fcut3))
    str_umin1 = meta1["Window"]["umin"]
    str_umin2 = meta2["Window"]["umin"]
    str_umin3 = meta3["Window"]["umin"]
    umin1 = eval(Meta.parse(str_umin1))
    umin2 = eval(Meta.parse(str_umin2))
    umin3 = eval(Meta.parse(str_umin3))
    str_umax1 = meta1["Window"]["umax"]
    str_umax2 = meta2["Window"]["umax"]
    str_umax3 = meta3["Window"]["umax"]
    umax1 = eval(Meta.parse(str_umax1))
    umax2 = eval(Meta.parse(str_umax2))
    umax3 = eval(Meta.parse(str_umax3))


    # fourier transforms
    ft_Psi41 = rfft(Psi41)
    ft_Psi42 = rfft(Psi42)
    ft_Psi43 = rfft(Psi43)
    ft_dhdt1 = rfft(dhdt1)
    ft_dhdt2 = rfft(dhdt2)
    ft_dhdt3 = rfft(dhdt3)
    ft_h1 = rfft(h1)
    ft_h2 = rfft(h2)
    ft_h3 = rfft(h3)
    ft_nobw_dhdt1 = rfft(nobw_dhdt1)
    ft_nobw_dhdt2 = rfft(nobw_dhdt2)
    ft_nobw_dhdt3 = rfft(nobw_dhdt3)
    ft_nobw_h1 = rfft(nobw_h1)
    ft_nobw_h2 = rfft(nobw_h2)
    ft_nobw_h3 = rfft(nobw_h3)
    f_Psi41 = rfftfreq(length(u_Psi41),1/(u_Psi41[2]-u_Psi41[1]))
    f_Psi42 = rfftfreq(length(u_Psi42),1/(u_Psi42[2]-u_Psi42[1]))
    f_Psi43 = rfftfreq(length(u_Psi43),1/(u_Psi43[2]-u_Psi43[1]))
    f_dhdt1 = rfftfreq(length(u_dhdt1),1/(u_dhdt1[2]-u_dhdt1[1]))
    f_dhdt2 = rfftfreq(length(u_dhdt2),1/(u_dhdt2[2]-u_dhdt2[1]))
    f_dhdt3 = rfftfreq(length(u_dhdt3),1/(u_dhdt3[2]-u_dhdt3[1]))
    f_h1 = rfftfreq(length(u_h1),1/(u_h1[2]-u_h1[1]))
    f_h2 = rfftfreq(length(u_h2),1/(u_h2[2]-u_h2[1]))
    f_h3 = rfftfreq(length(u_h3),1/(u_h3[2]-u_h3[1]))


    # maximum in power spectrum
    _, idxmax1 = findmax(abs.(ft_Psi41))
    _, idxmax2 = findmax(abs.(ft_Psi42))
    _, idxmax3 = findmax(abs.(ft_Psi43))
    f_Psi41_max = f_Psi41[idxmax1]
    f_Psi42_max = f_Psi42[idxmax2]
    f_Psi43_max = f_Psi43[idxmax3]
    f_Psi4_max = (f_Psi41_max + f_Psi42_max + f_Psi43_max) / 3


    fig = Figure()
    g0 = fig[1,0] = GridLayout()
    g1 = fig[1,1] = GridLayout()
    g2 = fig[1,2] = GridLayout()
    g3 = fig[1,3] = GridLayout()
    colsize!(fig.layout, 0, Relative(1/100))
    colsize!(fig.layout, 1, Relative(2/3))
    colsize!(fig.layout, 3, Relative(1/100))
    ax_Psi4     = Axis(g1[1,1], xlabel=L"u/M_{\textrm{BBS}}")
    ax_dhdt     = Axis(g1[2,1], xlabel=L"u/M_{\textrm{BBS}}")
    ax_h        = Axis(g1[3,1], xlabel=L"u/M_{\textrm{BBS}}")
    Label(g0[1,0], L"M_{\textrm{BBS}}~\text{Re}(r_{\infty}\Psi_{4,20})", rotation=pi/2, tellwidth=false, tellheight=false, width=nothing)
    Label(g0[2,0], L"\text{Re}(r_{\infty}dh/dt)", rotation=pi/2, tellwidth=false, tellheight=false, width=nothing)
    Label(g0[3,0], L"\text{Re}(r_{\infty}h_{4,20})/M_{\textrm{BBS}}", rotation=pi/2, tellheight=false, tellwidth=false, width=nothing)
    ax_psd_Psi4 = Axis(g2[1,1], xlabel=L"\log_{10}(f)", yaxisposition=:right)
    ax_psd_dhdt = Axis(g2[2,1], xlabel=L"\log_{10}(f)", yaxisposition=:right)
    ax_psd_h    = Axis(g2[3,1], xlabel=L"\log_{10}(f)", yaxisposition=:right)
    Label(g3[1,0], L"\text{PSD}(M_{\textrm{BBS}}r_{\infty}\Psi_{4,20})", rotation=pi/2, tellwidth=false, tellheight=false, width=nothing)
    Label(g3[2,0], L"\text{PSD}(r_{\infty}dh/dt)", rotation=pi/2, tellwidth=false, tellheight=false, width=nothing)
    Label(g3[3,0], L"\text{PSD}(r_{\infty}h_{4,20}/M_{\textrm{BBS}})", rotation=pi/2, tellheight=false, tellwidth=false, width=nothing)
    hidexdecorations!(ax_Psi4, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(ax_dhdt, ticks=false, grid=false, minorgrid=false, minorticks=false)
    hidexdecorations!(ax_psd_Psi4, ticks=false, grid=false)
    hidexdecorations!(ax_psd_dhdt, ticks=false, grid=false)
    rowgap!(g1, 0)
    rowgap!(g2, 0)


    lines!(ax_Psi4, u_Psi41./M, real.(Psi41.*M), label=L"f_{\textrm{cut}} = %$str_fcut1")
    lines!(ax_Psi4, u_Psi42./M, real.(Psi42.*M), label=L"f_{\textrm{cut}} = %$str_fcut2")
    lines!(ax_Psi4, u_Psi43./M, real.(Psi43.*M), label=L"f_{\textrm{cut}} = %$str_fcut3")

    lines!(ax_dhdt, u_dhdt1./M, real.(dhdt1))
    lines!(ax_dhdt, u_dhdt2./M, real.(dhdt2))
    lines!(ax_dhdt, u_dhdt3./M, real.(dhdt3))
    lines!(ax_dhdt, u_dhdt1./M, real.(nobw_dhdt1), linestyle=:dash, color=Makie.wong_colors()[1])
    lines!(ax_dhdt, u_dhdt2./M, real.(nobw_dhdt2), linestyle=:dash, color=Makie.wong_colors()[2])
    lines!(ax_dhdt, u_dhdt3./M, real.(nobw_dhdt3), linestyle=:dash, color=Makie.wong_colors()[3])

    lines!(ax_h, u_h1./M, real.(h1./M))
    lines!(ax_h, u_h2./M, real.(h2./M))
    lines!(ax_h, u_h3./M, real.(h3./M))
    lines!(ax_h, u_h1./M, real.(nobw_h1./M), linestyle=:dash, color=Makie.wong_colors()[1])
    lines!(ax_h, u_h2./M, real.(nobw_h2./M), linestyle=:dash, color=Makie.wong_colors()[2])
    lines!(ax_h, u_h3./M, real.(nobw_h3./M), linestyle=:dash, color=Makie.wong_colors()[3])

    lines!(ax_psd_Psi4, trafo.(f_Psi41), trafo.(abs.(ft_Psi41./M).^2))
    lines!(ax_psd_Psi4, trafo.(f_Psi42), trafo.(abs.(ft_Psi42./M).^2))
    lines!(ax_psd_Psi4, trafo.(f_Psi43), trafo.(abs.(ft_Psi43./M).^2))

    lines!(ax_psd_dhdt, trafo.(f_dhdt1), trafo.(abs.(ft_dhdt1).^2))
    lines!(ax_psd_dhdt, trafo.(f_dhdt2), trafo.(abs.(ft_dhdt2).^2))
    lines!(ax_psd_dhdt, trafo.(f_dhdt3), trafo.(abs.(ft_dhdt3).^2))
    lines!(ax_psd_dhdt, trafo.(f_dhdt1), trafo.(abs.(ft_nobw_dhdt1).^2), linestyle=:dash, color=Makie.wong_colors()[1])
    lines!(ax_psd_dhdt, trafo.(f_dhdt2), trafo.(abs.(ft_nobw_dhdt2).^2), linestyle=:dash, color=Makie.wong_colors()[2])
    lines!(ax_psd_dhdt, trafo.(f_dhdt3), trafo.(abs.(ft_nobw_dhdt3).^2), linestyle=:dash, color=Makie.wong_colors()[3])

    lines!(ax_psd_h, trafo.(f_h1), trafo.(abs.(ft_h1.*M).^2))
    lines!(ax_psd_h, trafo.(f_h2), trafo.(abs.(ft_h2.*M).^2))
    lines!(ax_psd_h, trafo.(f_h3), trafo.(abs.(ft_h3.*M).^2))
    lines!(ax_psd_h, trafo.(f_h1), trafo.(abs.(ft_nobw_h1.*M).^2), linestyle=:dash, color=Makie.wong_colors()[1])
    lines!(ax_psd_h, trafo.(f_h2), trafo.(abs.(ft_nobw_h2.*M).^2), linestyle=:dash, color=Makie.wong_colors()[2])
    lines!(ax_psd_h, trafo.(f_h3), trafo.(abs.(ft_nobw_h3.*M).^2), linestyle=:dash, color=Makie.wong_colors()[3])


    colors = Makie.wong_colors()
    lw = 3
    vlines!(ax_psd_Psi4, trafo(fcut1), color=colors[1], linestyle=:dashdot, linewidth=lw)
    vlines!(ax_psd_Psi4, trafo(fcut2), color=colors[2], linestyle=:dashdot, linewidth=lw)
    vlines!(ax_psd_Psi4, trafo(fcut3), color=colors[3], linestyle=:dashdot, linewidth=lw)

    vlines!(ax_psd_dhdt, trafo(fcut1), color=colors[1], linestyle=:dashdot, linewidth=lw)
    vlines!(ax_psd_dhdt, trafo(fcut2), color=colors[2], linestyle=:dashdot, linewidth=lw)
    vlines!(ax_psd_dhdt, trafo(fcut3), color=colors[3], linestyle=:dashdot, linewidth=lw)

    vlines!(ax_psd_h, trafo(fcut1), color=colors[1], linestyle=:dashdot, linewidth=lw)
    vlines!(ax_psd_h, trafo(fcut2), color=colors[2], linestyle=:dashdot, linewidth=lw)
    vlines!(ax_psd_h, trafo(fcut3), color=colors[3], linestyle=:dashdot, linewidth=lw)

    vlines!(ax_psd_Psi4, trafo(f_Psi4_max), color=:black, linestyle=:dashdotdot, linewidth=lw)
    vlines!(ax_psd_dhdt, trafo(f_Psi4_max), color=:black, linestyle=:dashdotdot, linewidth=lw)
    vlines!(ax_psd_h,    trafo(f_Psi4_max), color=:black, linestyle=:dashdotdot, linewidth=lw)

    linkxaxes!(ax_psd_Psi4, ax_psd_dhdt, ax_psd_h)
    linkyaxes!(ax_psd_Psi4, ax_psd_dhdt, ax_psd_h)
    xlims!(ax_psd_h, (-3.3,-0.7))
    ylims!(ax_psd_h, (-14,nothing))
    ax_psd_h.xticks[] = -3:0

    Legend(fig[0,1:2], ax_Psi4, width=nothing, tellwidth=false,
           orientation=:horizontal, colgap=35)#, labelsize=25)

    # freeze axis limits; hacky way, from discord question
    reset_limits!(ax_Psi4)
    reset_limits!(ax_dhdt)
    reset_limits!(ax_h)
    limits!(ax_Psi4, ax_Psi4.finallimits[])
    limits!(ax_dhdt, ax_dhdt.finallimits[])
    limits!(ax_h, ax_h.finallimits[])

    # draw shaded regions to indcate windowing range
    low1, up1 = [ Point2f(-1000, -1000), Point2f(-1000,1000) ], [ Point2f(umin1, -1000), Point2f(umin1, 1000) ]
    band!(ax_Psi4, low1./M, up1./M, color=(:gray,0.3))
    band!(ax_dhdt, low1./M, up1./M, color=(:gray,0.3))
    band!(ax_h,    low1./M, up1./M, color=(:gray,0.3))
    low1, up1 = [ Point2f(umax1, -1000), Point2f(umax1,1000) ], [ Point2f(10000, -1000), Point2f(10000, 1000) ]
    band!(ax_Psi4, low1./M, up1./M, color=(:gray,0.3))
    band!(ax_dhdt, low1./M, up1./M, color=(:gray,0.3))
    band!(ax_h,    low1./M, up1./M, color=(:gray,0.3))

    # fig

    @info "Saving as '$plotname'"
    save(plotname, fig, resolution=(2400,1200), backend=CairoMakie)

end


end
