using GWanalysis
using DataFrames
using Printf
using GLMakie
GLMakie.activate!()


include("../theme.jl")
include("../rootdirs.jl")
include("utils.jl")


let

# bare scalar field mass
mu = 1e-11
# frequency window for mismatch integral
fmin, fmax = 6, 1000 # Hz

runs = find_runs(OUTPUTS["comparison8/naive_v_0.1"])

df = DataFrame(:output => BampsOutput.(runs))

GWanalysis.getparameter(df, Int64, "grid.n.xyz"; newkey="n")
@assert allunique(df."n")
sort!(df, :n)

# load all data
GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_1")
GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_2")
GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_3")
GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_4")
GWanalysis.get_output_analysis2(df, "Psi4_20_Nextrp_5")

# noise whiten 2nd differenced psi4
f_1, dd_ft_1 = compute_dd_psi4(df."Psi4_20_Nextrp_1:Time", df."Psi4_20_Nextrp_1:Psi4")
f_2, dd_ft_2 = compute_dd_psi4(df."Psi4_20_Nextrp_2:Time", df."Psi4_20_Nextrp_2:Psi4")
f_3, dd_ft_3 = compute_dd_psi4(df."Psi4_20_Nextrp_3:Time", df."Psi4_20_Nextrp_3:Psi4")
f_4, dd_ft_4 = compute_dd_psi4(df."Psi4_20_Nextrp_4:Time", df."Psi4_20_Nextrp_4:Psi4")
f_5, dd_ft_5 = compute_dd_psi4(df."Psi4_20_Nextrp_5:Time", df."Psi4_20_Nextrp_5:Psi4")

psd_name = "T1800044-v5.txt"
f_psd, psd = GWanalysis.load_psd(psd_name)
# compute 2nd differenced noise curve
dd_psd = compute_dd_psd(f_psd, psd)
# limit noise curve to (fmin,fmax) window
idx_fmin = findfirst(>=(fmin), f_psd) |> something
idx_fmax = findfirst(>=(fmax), f_psd) |> something
f_psd    = f_psd[idx_fmin:idx_fmax-1]
psd      = psd[idx_fmin:idx_fmax-1]
dd_psd   = dd_psd[idx_fmin:idx_fmax-1]

mms_1 = compute_psi4_wienerprod_series(f_1, dd_ft_1, f_psd, dd_psd, mu)
mms_2 = compute_psi4_wienerprod_series(f_2, dd_ft_2, f_psd, dd_psd, mu)
mms_3 = compute_psi4_wienerprod_series(f_3, dd_ft_3, f_psd, dd_psd, mu)
mms_4 = compute_psi4_wienerprod_series(f_4, dd_ft_4, f_psd, dd_psd, mu)
mms_5 = compute_psi4_wienerprod_series(f_5, dd_ft_5, f_psd, dd_psd, mu)

ns = df.n

fig = Figure()
ax = Axis(fig[1,1],
          xlabel=L"n_{i+1} - n_{i}",
          ylabel=L"\log_{10}(1 - \mathcal{W})")
xticks = [ @sprintf("%d - %d",ns[i],ns[i+1]) for i = 1:length(ns)-1 ]
ax.xticks[] = (1:length(xticks), xticks)
scatterlines!(ax, 1:length(xticks), log10.(mms_1), label=L"N_{\textrm{extrp}} = 1")
scatterlines!(ax, 1:length(xticks), log10.(mms_2), label=L"N_{\textrm{extrp}} = 2")
scatterlines!(ax, 1:length(xticks), log10.(mms_3), label=L"N_{\textrm{extrp}} = 3")
scatterlines!(ax, 1:length(xticks), log10.(mms_4), label=L"N_{\textrm{extrp}} = 4")
scatterlines!(ax, 1:length(xticks), log10.(mms_5), label=L"N_{\textrm{extrp}} = 5")
axislegend(ax)
display(fig)

end
